Package: TCGAbiolinks
Type: Package
Title: TCGAbiolinks: An R/Bioconductor package for integrative analysis
        with GDC data
Version: 2.34.1
Date: 2024-01-01
Author: Antonio Colaprico,
    Tiago Chedraoui Silva,
    Catharina Olsen,
    Luciano Garofano,
    Davide Garolini,
    Claudia Cava,
    Thais Sabedot,
    Tathiane Malta,
    Stefano M. Pagnotta,
    Isabella Castiglioni,
    Michele Ceccarelli,
    Gianluca Bontempi,
    Houtan Noushmehr
Maintainer: Tiago Chedraoui Silva <tiagochst@gmail.com>,
 Antonio Colaprico <axc1833@med.miami.edu>
Depends: R (>= 4.0)
Imports: downloader (>= 0.4), grDevices, biomaRt, dplyr, graphics,
        tibble, GenomicRanges, XML (>= 3.98.0), data.table, jsonlite
        (>= 1.0.0), plyr, knitr, methods, ggplot2, stringr (>= 1.0.0),
        IRanges, rvest (>= 0.3.0), stats, utils, S4Vectors, R.utils,
        SummarizedExperiment (>= 1.4.0), TCGAbiolinksGUI.data (>=
        1.15.1), readr, tools, tidyr, purrr, xml2, httr (>= 1.2.1)
Description: The aim of TCGAbiolinks is : i) facilitate the GDC open-access
    data retrieval, ii) prepare the data using the appropriate pre-processing
    strategies, iii) provide the means to carry out different standard analyses
    and iv) to easily reproduce earlier research results. In more detail, the package
    provides multiple methods for analysis (e.g., differential expression analysis,
    identifying differentially methylated regions) and methods for visualization
    (e.g., survival plots, volcano plots, starburst plots) in order to easily
    develop complete analysis pipelines.
License: GPL (>= 3)
biocViews: DNAMethylation, DifferentialMethylation, GeneRegulation,
        GeneExpression, MethylationArray, DifferentialExpression,
        Pathways, Network, Sequencing, Survival, Software
Suggests: jpeg, png, BiocStyle, rmarkdown, devtools, maftools,
        parmigene, c3net, minet, Biobase, affy, testthat, sesame,
        AnnotationHub, ExperimentHub, pathview, clusterProfiler,
        Seurat, ComplexHeatmap, circlize, ConsensusClusterPlus, igraph,
        supraHex, limma, edgeR, sva, EDASeq, survminer, genefilter,
        gridExtra, survival, doParallel, parallel, ggrepel (>= 0.6.3),
        scales, grid, DT
VignetteBuilder: knitr
LazyData: true
URL: https://github.com/BioinformaticsFMRP/TCGAbiolinks
BugReports: https://github.com/BioinformaticsFMRP/TCGAbiolinks/issues
RoxygenNote: 7.3.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/TCGAbiolinks
git_branch: RELEASE_3_20
git_last_commit: 93c796a4
git_last_commit_date: 2025-03-06
Repository: Bioconductor 3.20
Date/Publication: 2025-03-06
NeedsCompilation: no
Packaged: 2025-03-07 00:57:31 UTC; biocbuild
