#' TCGA samples with their Pam50 subtypes
#'
#' A dataset containing the Sample Ids from TCGA and PAM50 subtyping attributes of 4768 tumor patients
#'
#' @format A data frame with 4768 rows and 3 variables:
#' \describe{
#'   \item{samples}{Sample ID from TCGA barcodes, character string}
#'   \item{subtype}{Pam50 classification, character string}
#'   \item{color}{color, character string}
#'   ...
#' }

"TabSubtypesCol_merged"
