## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_knit$set(progress = FALSE)

## ---- eval = FALSE------------------------------------------------------------
#  if (!requireNamespace("BiocManager", quietly=TRUE))
#      install.packages("BiocManager")
#  BiocManager::install("TCGAbiolinksGUI", dependencies = TRUE)

## ---- eval = FALSE------------------------------------------------------------
#  if (!requireNamespace("BiocManager", quietly=TRUE))
#      install.packages("BiocManager")
#  devtools::install_github("BioinformaticsFMRP/TCGAbiolinksGUI.data",ref = "R_3.4")
#  devtools::install_github("BioinformaticsFMRP/TCGAbiolinksGUI")

## ---- eval = FALSE------------------------------------------------------------
#  library(TCGAbiolinksGUI)
#  TCGAbiolinksGUI()

