% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{GenesCutID}
\alias{GenesCutID}
\title{GenesCutID}
\usage{
GenesCutID(GeneList)
}
\arguments{
\item{GeneList}{GeneList}
}
\value{
list of gene symbol without IDs
}
\description{
GenesCutID
}
\keyword{internal}
