% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TCGAbiolinks.R
\docType{data}
\name{msi_results}
\alias{msi_results}
\title{MSI data for two samples}
\format{
A data frame: 2 rows, 4 columns
}
\description{
MSI data for two samples
}
\keyword{internal}
