% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TCGAbiolinks.R
\docType{data}
\name{clinBRCA}
\alias{clinBRCA}
\title{Clinical data TCGA BRCA}
\format{
A data frame with 1061 rows and 109 variables
}
\description{
Clinical data TCGA BRCA
}
\keyword{internal}
