% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{getGDCprojects}
\alias{getGDCprojects}
\title{Retrieve all GDC projects}
\usage{
getGDCprojects()
}
\value{
A data frame with last GDC projects
}
\description{
getGDCprojects uses the following api to get projects
  https://api.gdc.cancer.gov/projects
}
\examples{
projects <- getGDCprojects()
}
